/*
 * A page queue abstraction.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <list>
#include <map>

#include "flexpage.h"
#include "memory.h"
#include "page_owner.h"



/* Collection types. */

typedef struct { Flexpage *flexpage; PageOwner *owner; } QueueEntry;
typedef std::list<QueueEntry> Queue;

typedef std::pair<Flexpage *, Queue::iterator> Position;
typedef std::map<Flexpage *, Queue::iterator> Positions;



/* A queue of managed pages. */

class PageQueue
{
protected:

    /* Helper methods. */

    virtual void discard(Queue &queue, Memory *memory);

    virtual bool remove(Queue &queue, Positions &positions, PageOwner *owner, Flexpage *flexpage);

public:
    virtual ~PageQueue()
    {
    }

    virtual void close(Memory *memory) = 0;

    virtual void pop(PageOwner **owner, Flexpage **flexpage) = 0;

    virtual void push(PageOwner *owner, Flexpage *flexpage) = 0;

    virtual void push_front(PageOwner *owner, Flexpage *flexpage) = 0;

    virtual bool remove(PageOwner *owner, Flexpage *flexpage) = 0;
};

// vim: tabstop=4 expandtab shiftwidth=4
